<?php
/* --------------------------------------------------------------
   CLIHelper.inc.php 2016-09-02
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2016 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class CLIHelper
 */
class CLIHelper
{
	/**
	 * Echoes a string and adds a Timestamp and a linebreak
	 *
	 * @param string $string
	 */
	static public function doLog($string = '')
	{
		echo '[' . date('Y-m-d H:i:s') . '] ' . $string . "\n";
	}
	
	
	/**
	 * Echoes a string and adds a Timestamp and a linebreak and executes it via system()
	 *
	 * @param string $string
	 */
	static public function doSystem($string = '')
	{
		self::doLog($string);
		system($string);
	}
	
	
	/**
	 * Reports SQL errors and terminates the script, if any SQL errors occurred during the update process
	 *
	 * @param DatabaseModel $update
	 */
	static public function reportUpdateDBErrors(DatabaseModel $update)
	{
		$sqlErrors = $update->get_sql_errors();
		if(count($sqlErrors) !== 0)
		{
			foreach($sqlErrors as $error)
			{
				self::doLog($error['error']);
				self::doLog('Query:');
				self::doLog($error['query']);
			}
			
			exit(0);
		}
	}
	
	
	/**
	 * Transforms parameters (commandline or GET parameters) into a normalized array
	 *
	 * @param array|null $commandlineArguments
	 *
	 * @return array
	 */
	static public function getParameters(array $commandlineArguments = null)
	{
		return $_GET ? : self::_mapArguments($commandlineArguments) ? : [];
	}
	
	
	/**
	 * Maps all expected commandline arguments
	 *
	 * @param array $arguments Given commandline arguments
	 *
	 * @return array All expected parameters
	 */
	static protected function _mapArguments(array $arguments)
	{
		self::doLog('Mapping arguments...');
		$parameters      = [];
		$argumentMapping = [
			1 => 'security_token'
		];
		
		foreach($argumentMapping as $index => $parameterKey)
		{
			$parameters[$parameterKey] = array_key_exists($index, $arguments) ? $arguments[$index] : '';
		}
		
		return $parameters;
	}
	
	
	/**
	 * Returns a GambioUpdateControl instance
	 *
	 * @return \GambioUpdateControl
	 */
	static public function getGambioUpdateControl()
	{
		self::doLog('Loading available updates...');
		$updateControl = new GambioUpdateControl(DB_SERVER, DB_SERVER_USERNAME, DB_SERVER_PASSWORD, DB_DATABASE);
		
		return $updateControl;
	}
	
	
	/**
	 * Gets and returns an array of all given commandline options
	 *
	 * @return array Commandline options
	 */
	static public function getOptions()
	{
		self::doLog('Gathering commandline options...');
		$shortOptions = '';
		$longOptions  = [
			'help'
		];
		
		return getopt($shortOptions, $longOptions) ? : [];
	}
	
	
	/**
	 * Proceeds options
	 *
	 * @param array $options
	 */
	static public function proceedOptions(array $options)
	{
		self::doLog('Proceeding commandline options...');
		if(array_key_exists('help', $options))
		{
			self::doLog('HELP!');
		}
	}
	
	
	/**
	 * Authenticates an admin by a given security token.
	 *
	 * @param string $token The security token
	 *
	 * @return bool Indicates if authentication failed or succeeded
	 */
	static public function authenticateAdmin($token = '')
	{
		self::doLog('Authenticating admin...');
		if(UpdaterLogin::auth($token) === false)
		{
			self::doLog('Permission denied.');
			exit(0);
		}
	}
	
	
	/**
	 * Processes the available, compatible updates
	 *
	 * @param \GambioUpdateControl $updateControl
	 */
	static public function processUpdates(GambioUpdateControl $updateControl)
	{
		self::doLog('Processing updates...');
		foreach($updateControl->gambio_update_array as $update)
		{
			self::doLog($update->get_name() . ':');
			self::_executeUpdateDependent($update);
			self::_executeUpdateIndependent($update);
			self::_executeUpdateCSS($update);
			self::_executeUpdateVersionHistory($update);
		}
	}
	
	
	/**
	 * Executes all dependent DB operations of a given update
	 *
	 * @param \GambioUpdateModel $update
	 */
	static protected function _executeUpdateDependent(GambioUpdateModel $update)
	{
		self::doLog('...dependent');
		$update->update_dependent_data();
		self::reportUpdateDBErrors($update);
	}
	
	
	/**
	 * Executes all independent DB operations of a given update
	 *
	 * @param \GambioUpdateModel $update
	 */
	static protected function _executeUpdateIndependent(GambioUpdateModel $update)
	{
		self::doLog('...independent');
		$update->update_independent_data();
		self::reportUpdateDBErrors($update);
	}
	
	
	/**
	 * Updates CSS changes (EyeCandy only) of a given update
	 *
	 * @param \GambioUpdateModel $update
	 */
	static protected function _executeUpdateCSS(GambioUpdateModel $update)
	{
		self::doLog('...CSS');
		$update->update_css();
		self::reportUpdateDBErrors($update);
	}
	
	
	/**
	 * Sets the current shop version to the version of the given update
	 *
	 * @param \GambioUpdateModel $update
	 */
	static protected function _executeUpdateVersionHistory(GambioUpdateModel $update)
	{
		self::doLog('...version history');
		$update->update_version_history();
		self::reportUpdateDBErrors($update);
	}
	
	
	/**
	 * Executes the file movements for all available, compatible updates
	 *
	 * @param \GambioUpdateControl $updateControl
	 */
	static public function moveFiles(GambioUpdateControl $updateControl)
	{
		self::doLog('Moving files...');
		FilesystemManager::move($updateControl->get_move_array());
	}
	
	
	/**
	 * Executes the file deletions for all available, compatible updates
	 *
	 * @param \GambioUpdateControl $updateControl
	 */
	static public function deleteFiles(GambioUpdateControl $updateControl)
	{
		self::doLog('Deleting files...');
		FilesystemManager::delete(array_filter($updateControl->get_delete_list(), function ($value)
		{
			return trim($value) !== '';
		}));
	}
	
	
	/**
	 * Checks and sets full file permissions according to the files that are pointed at by the given file paths
	 *
	 * @param string $singleChmodFilePath    File path to a list of single files and directories
	 * @param string $recursiveChmodFilePath File path to a list of directories that should be processed recursively
	 */
	static public function changeFilePermissions($singleChmodFilePath = '', $recursiveChmodFilePath = '')
	{
		self::doLog('Changing file permissions...');
		if($singleChmodFilePath !== '' && file_exists($singleChmodFilePath))
		{
			FilesystemManager::singleChmod($singleChmodFilePath);
		}
		
		self::doLog('Changing file permissions recursively...');
		if($recursiveChmodFilePath !== '' && file_exists($recursiveChmodFilePath))
		{
			FilesystemManager::recursiveChmod($recursiveChmodFilePath);
		}
	}
	
	
	/**
	 * Clears all caches and rebuilds index tables
	 * 
	 * @param \GambioUpdateControl $updateControl
	 */
	static public function clearCache(GambioUpdateControl $updateControl)
	{
		self::doLog('Clearing caches...');
		xtc_db_connect() or die('Unable to connect to database server!');
		$updateControl->clear_cache();
		
		self::doLog('Rebuilding caches...');
		xtc_db_connect() or die('Unable to connect to database server!');
		chdir(DIR_FS_CATALOG);
		$updateControl->rebuild_cache();
	}
	
	
	/**
	 * Sets the configuration value 'INSTALLED_VERSION' to the value contained in the release_info.php
	 *
	 * @param \GambioUpdateControl $updateControl
	 */
	static public function setInstalledVersion(GambioUpdateControl $updateControl)
	{
		self::doLog('Setting INSTALLED_VERSION configuration...');
		$updateControl->set_installed_version();
	}
}